# -*- coding: utf-8 -*-

from ladon.interfaces.base import BaseInterface,ServiceDescriptor,BaseRequestHandler,BaseResponseHandler
from ladon.interfaces import expose
from ladon.compat import type_to_jsontype,pytype_support,PORTABLE_STRING
import json,sys

class JSONWSPServiceDescriptor(ServiceDescriptor):
	
	javascript_type_map = type_to_jsontype
	version = '1.0'
	_content_type = 'application/json'

	def generate(self,servicename,servicenumber,typemanager,methodlist,service_url,encoding):
		type_dict = typemanager.type_dict
		type_order = typemanager.type_order

		def map_type(typ):
			if typ in JSONWSPServiceDescriptor.javascript_type_map:
				return JSONWSPServiceDescriptor.javascript_type_map[typ]
			else:
				return typ.__name__

		desc = {
			'servicename': servicename,
			'url': service_url,
			'type': 'jsonwsp/description',
			'version': self.version,
			'types': {},
			'methods': {}
		}
		
		types = desc['types']
		for typ in type_order:
			if isinstance(typ, dict):
				desc_type = {}
				types[typ['name']] = desc_type
				for k,v in typ['attributes']:
					if isinstance(v, list):
						desc_type_val = [map_type(v[0])]
					else:
						desc_type_val = map_type(v)
					desc_type[k] = desc_type_val

		methods = desc['methods']
		for m in methodlist:
			desc_mparams = {}
			order = 1
			desc_method = {'params': desc_mparams, 'doc_lines': m._method_doc}
			methods[m.name()] = desc_method
			for arg in m.args():
				if isinstance(arg['type'], (list, tuple)):
					desc_param_type = [map_type(arg['type'][0])]
				else:
					desc_param_type = map_type(arg['type'])
				desc_mparams[arg['name']] = {
					"type": desc_param_type,
					"def_order": order,
					"optional": arg['optional'],
					}
				if 'doc' in arg:
					desc_mparams[arg['name']]["doc_lines"] = arg['doc']
				else:
					desc_mparams[arg['name']]["doc_lines"] = []
				order += 1
				
			if isinstance(m._rtype, (list, tuple)):
				desc_rtype = [map_type(m._rtype[0])]
			else:
				desc_rtype = map_type(m._rtype)
			desc_method['ret_info'] = {
				'type': desc_rtype,
				'doc_lines': m._rtype_doc
			}
			
		if sys.version_info < (3, 0):
			return json.dumps(desc, encoding=encoding)
		return json.dumps(desc)


class JSONWSPRequestHandler(BaseRequestHandler):
	
	def parse_request(self,json_body,sinfo,encoding):
		def parse_number(x):
			return PORTABLE_STRING(x)
		def parse_constant(x):
			if x=='false':
				return PORTABLE_STRING("False")
			elif x=='true':
				return PORTABLE_STRING("True")
			elif x=='null':
				return PORTABLE_STRING("None")
			return PORTABLE_STRING(x)
		req_dict = json.loads(PORTABLE_STRING(json_body,encoding),parse_int=parse_number,parse_float=parse_number,parse_constant=parse_constant)
		return req_dict



class JSONWSPResponseHandler(BaseResponseHandler):
	
	_content_type = 'application/json'
	_stringify_res_dict = False
	version = '1.0'
	
	def build_response(self,res_dict,sinfo,encoding):
		res_dict['type'] = 'jsonwsp/response'
		res_dict['version'] = self.version
		res_dict['methodname'] = res_dict['method']
		del res_dict['method']
		return json.dumps(res_dict,ensure_ascii=False).encode(encoding)


@expose
class JSONWSPInterface(BaseInterface):

	def __init__(self,sinfo,**kw):
		def_kw = {
			'service_descriptor': JSONWSPServiceDescriptor,
			'request_handler': JSONWSPRequestHandler,
			'response_handler': JSONWSPResponseHandler}
		def_kw.update(kw)
		BaseInterface.__init__(self,sinfo,**def_kw)

	@staticmethod
	def _interface_name():
		return 'jsonwsp'

	@staticmethod
	def _accept_basetype(typ):
		return pytype_support.count(typ)>0

	@staticmethod
	def _accept_list():
		return True

	@staticmethod
	def _accept_dict():
		return False

