# -*- coding: utf-8 -*-
from threading import Thread
from wsgiref import simple_server, validate
import os

from ladon.server.wsgi import LadonWSGIApplication

service_modules = [
	'stringtests',
	'typetests',
	'attachmenttests'
]

def make_server():
	global service_modules
	application = LadonWSGIApplication(service_modules, [os.path.join(os.path.dirname(__file__), 'services')])

	# Wrap the application in WSGI validator
	validator_app = validate.validator(application)

	# Bind to localhost so we don't have to worry about firewall exceptions
	return simple_server.make_server('localhost', 2376, validator_app)

class ServiceThread(Thread):
	def __init__(self, *args, **kwargs):
		super(ServiceThread, self).__init__(*args, **kwargs)
		self.server = make_server()

	def run(self):
		self.server.serve_forever()

def serve_test_service(as_thread=False):
	if as_thread:
		t = ServiceThread()
		t.start()
		return t
	else:
		server = make_server()
		server.serve_forever()

if __name__=='__main__':
	serve_test_service()
